from typing import (
	Sequence,
	Union,
)

import numpy as np

from mergechannels import (
	dispatch_single_channel,
	dispatch_multi_channel,
)
from ._luts import COLORMAPS
from ._blending import BLENDING_OPTIONS

def apply_color_map(
	arr: np.ndarray,
	color: COLORMAPS,
	percentiles: Union[tuple[float, float], None] = None,
	saturation_limits: Union[tuple[float, float], None] = None,
) -> np.ndarray:
	'''
	apply a color map to an array
	'''
	if saturation_limits is None:
		if percentiles is None:
			percentiles = (1.1, 99.9)
		low, high = np.percentile(arr, percentiles)
		saturation_limits = (low, high)

	return dispatch_single_channel(
		array_reference=arr,
		cmap_name=color,
		limits=saturation_limits,
	)


def merge(
	arrs: Sequence[np.ndarray],
	colors: Sequence[COLORMAPS],
	blending: BLENDING_OPTIONS = 'max',
	percentiles: Union[Sequence[tuple[float, float]], None] = None,
	saturation_limits: Union[Sequence[tuple[float, float]], None] = None,
) -> np.ndarray:
	'''
	apply cmaps to arrays and blend the colors
	'''
	if saturation_limits is None:
		if percentiles is None:
			percentiles = [(1.1, 99.9)] * len(arrs)
		saturation_limits = tuple(
			np.percentile(arr, ch_percentiles)
			for arr, ch_percentiles in zip(arrs, percentiles)  # type: ignore
		)
	return dispatch_multi_channel(
		array_references=arrs,
		cmap_names=colors,
		blending=blending,
		limits=saturation_limits,  # type: ignore
	)



from typing import Literal

COLORMAPS = Literal[
	'Circus Ink Purple',
	'Spectrum Parabolic Ramp',
	'Pop cherry',
	'Circus Green',
	'Rainbow RGB',
	'Parabolic RGB',
	'3color-RMB',
	'Pop magenta inverted',
	'Red Hot',
	'Duo Cyan',
	'Yellow',
	'Grays g=0.25 inverted',
	'betterRed',
	'Grays g=1.75',
	'blue_orange_icb',
	'Pop green',
	'mpl-inferno',
	'I Blue',
	'BOP purple',
	'Duo Yellow',
	'Phase Ink Mint-Cherry',
	'betterOrange',
	'Blue',
	'6_shades',
	'Ink Cyan',
	'OIMB3',
	'Quartetto MYGB Blue',
	'Ink Green',
	'Red/Green',
	'Turbo',
	'Circus Ink Black',
	'Duo Green',
	'mpl-viridis',
	'Grays g=0.25',
	'Quartetto RYGB Blue',
	'smart',
	'Thermal',
	'OIMB2',
	'Pop blue inverted',
	'Duo Intense Cherry',
	'Phase Bold Green-Magenta',
	'Quartetto MYGB Magenta',
	'Pop purple inverted',
	'glow',
	'OPF orange',
	'Pop cyan inverted',
	'physics',
	'Phase Ink Yellow-Cyan',
	'Circus Ink Mint',
	'Ink Wash Purple',
	'OIMB1',
	'betterCyan',
	'mpl-magma',
	'Duo Intense Green',
	'edges',
	'Quartetto MYGB Green',
	'Ink Wash Cherry',
	'OPF purple',
	'Ink Wash Cyan',
	'Duo Intense Purple',
	'I Yellow',
	'16_colors',
	'Circus Cyan',
	'Pop lavender inverted',
	'sepia',
	'Duo Intense Cyan',
	'Grays g=1.00 inverted',
	'ICA2',
	'Quartetto RYGB Green',
	'Ink Wash Green',
	'Pop blue',
	'Quartetto RYGB Red',
	'I Forest',
	'ICA3',
	'Circus Yellow',
	'Pop mint',
	'BOP orange',
	'Grays g=1.25 inverted',
	'I Red',
	'3color-CGY',
	'I Green',
	'Fire',
	'Magenta Hot',
	'Grays g=0.50',
	'Orange Hot',
	'Circus Ink Cherry',
	'Pop purple',
	'Ink Yellow',
	'unionjack',
	'Ink Wash Yellow',
	'I Cyan',
	'Cyan Hot',
	'Pop mint inverted',
	'I Magenta',
	'Grays',
	'Phase Green-Magenta',
	'Cyan',
	'Grays g=0.75',
	'Phase Bold Ink Mint-Cherry',
	'Grays g=1.75 inverted',
	'phase',
	'gem',
	'Pop cherry inverted',
	'Pop red',
	'Circus Cherry',
	'OPF fresh',
	'Circus Ink Cyan',
	'betterGreen',
	'Phase Ink Green-Magenta',
	'3color-BMR',
	'Ink Wash Black',
	'Pop red inverted',
	'Duo Purple',
	'betterBlue',
	'Phase Bold Ink Yellow-Cyan',
	'Quartetto RYGB Yellow',
	'Magenta',
	'Duo Mint',
	'Red',
	'Grays g=1.25',
	'I Bordeaux',
	'Grays g=1.50 inverted',
	'Green Hot',
	'glasbey',
	'glasbey_on_dark',
	'mpl-plasma',
	'Ink Cherry',
	'Ink Mint',
	'Circus Ink Yellow',
	'Pop cyan',
	'Pop green inverted',
	'Ink Purple',
	'3color-YGC',
	'Pop yellow',
	'Quartetto MYGB Yellow',
	'Duo Cherry',
	'Green',
	'Grays g=0.50 inverted',
	'Circus Ink Green',
	'ICA',
	'Phase Mint-Cherry',
	'Ink Black',
	'Yellow Hot',
	'Phase Bold Yellow-Cyan',
	'thallium',
	'Pop yellow inverted',
	'Spectrum',
	'royal',
	'glasbey_inverted',
	'betterYellow',
	'Circus Purple',
	'Grays g=2.00',
	'HiLo',
	'cool',
	'Green Fire Blue',
	'Ink Wash Mint',
	'Grays g=0.75 inverted',
	'BOP blue',
	'thal',
	'Ice',
	'Circus Mint',
	'Grays g=1.50',
	'Duo Intense Mint',
	'brgbcmyw',
	'Phase Yellow-Cyan',
	'I Purple',
	'Duo Intense Yellow',
	'Phase Bold Ink Green-Magenta',
	'Pop magenta',
	'3-3-2 RGB',
	'Grays g=2.00 inverted',
	'5_ramps',
	'Phase Bold Mint-Cherry',
]




from typing import Literal, Sequence
import numpy as np

from python.mergechannels._blending import BLENDING_OPTIONS
from python.mergechannels._luts import COLORMAPS


def apply_color_map(
	arr: np.ndarray,
	color: COLORMAPS,
	percentiles: tuple[float, float] | None = None,
	saturation_limits: tuple[float, float] | None = None,
) -> np.ndarray:
    ...

def merge(
	arrs: Sequence[np.ndarray],
	colors: Sequence[COLORMAPS],
	blending: BLENDING_OPTIONS = 'max',
	percentiles: tuple[float, float] | Sequence[tuple[float, float]] | None = None,
	saturation_limits: tuple[float, float] | Sequence[tuple[float, float]] | None = None,
) -> np.ndarray:
    ...

def dispatch_single_channel(
    array_reference: np.ndarray,
    cmap_name: str,
    limits: tuple[float, float],
) -> np.ndarray:
    ...

def dispatch_multi_channel(
    array_references: Sequence[np.ndarray],
    cmap_names: Sequence[str],
    blending: Literal[BLENDING_OPTIONS],
    limits: Sequence[tuple[float, float]],
) -> np.ndarray:
    ...
